@@ -9,11 +9,13 @@ import (
 
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
+	"github.com/superfly/flyctl/client"
 	"github.com/superfly/flyctl/flaps"
 	"github.com/superfly/flyctl/internal/app"
 	"github.com/superfly/flyctl/internal/command"
+	"github.com/superfly/flyctl/internal/command/apps"
 	"github.com/superfly/flyctl/internal/flag"
-	"github.com/superfly/flyctl/iostreams"
+	"github.com/superfly/flyctl/internal/machine"
 )
 
 func newRestart() *cobra.Command {
@@ -49,77 +51,72 @@ func newRestart() *cobra.Command {
 			Name:        "force",
 			Description: "Force stop the machine(s)",
 		},
+		flag.Bool{
+			Name:        "skip-health-checks",
+			Description: "Restarts app without waiting for health checks. ( Machines only )",
+			Default:     false,
+		},
 	)
 
 	return cmd
 }
 
-func runMachineRestart(ctx context.Context) (err error) {
+func runMachineRestart(ctx context.Context) error {
 	var (
-		io      = iostreams.FromContext(ctx)
 		args    = flag.Args(ctx)
 		signal  = flag.GetString(ctx, "signal")
 		timeout = flag.GetInt(ctx, "time")
-	)
-
-	var forceStop = false
-
-	if flag.GetBool(ctx, "force") {
-		forceStop = true
-	}
-
-	for _, machineID := range args {
-		fmt.Fprintf(io.Out, "Sending kill signal to machine %s...", machineID)
-
-		if err = Restart(ctx, machineID, signal, timeout, forceStop); err != nil {
-			return
-		}
-		fmt.Fprintf(io.Out, "%s has been successfully stopped\n", machineID)
-	}
-	return
-}
-
-func Restart(ctx context.Context, machineID, sig string, timeOut int, forceStop bool) (err error) {
-	var (
 		appName = app.NameFromContext(ctx)
+		client  = client.FromContext(ctx).API()
 	)
 
-	input := api.RestartMachineInput{
-		ID:        machineID,
-		Timeout:   time.Duration(timeOut),
-		ForceStop: forceStop,
-	}
-
-	if sig != "" {
-		signal := &api.Signal{}
-
-		s, err := strconv.Atoi(sig)
-		if err != nil {
-			return fmt.Errorf("could not get signal %s", err)
-		}
-		signal.Signal = syscall.Signal(s)
-		input.Signal = signal
-	}
-
-	app, err := appFromMachineOrName(ctx, machineID, appName)
+	app, err := client.GetAppCompact(ctx, appName)
 	if err != nil {
 		return fmt.Errorf("could not get app: %w", err)
 	}
 
-	flapsClient, err := flaps.New(ctx, app)
+	ctx, err = apps.BuildContext(ctx, app)
 	if err != nil {
-		return fmt.Errorf("could not make flaps client: %w", err)
+		return err
 	}
 
-	err = flapsClient.Restart(ctx, input)
-	if err != nil {
-		return fmt.Errorf("could not stop machine %s: %w", input.ID, err)
+	// Resolve flags
+	input := &api.RestartMachineInput{
+		Timeout:          time.Duration(timeout),
+		ForceStop:        flag.GetBool(ctx, "force"),
+		SkipHealthChecks: flag.GetBool(ctx, "skip-health-checks"),
 	}
 
-	ctx = flaps.NewContext(ctx, flapsClient)
-	if err = WaitForStartOrStop(ctx, &api.Machine{ID: input.ID}, "start", time.Minute*5); err != nil {
-		return
+	if signal != "" {
+		sig := &api.Signal{}
+
+		s, err := strconv.Atoi(flag.GetString(ctx, "signal"))
+		if err != nil {
+			return fmt.Errorf("could not get signal %s", err)
+		}
+		sig.Signal = syscall.Signal(s)
+		input.Signal = sig
 	}
 
-	return
+	flapsClient := flaps.FromContext(ctx)
+
+	// Restart each machine
+	for _, machineID := range args {
+		m, err := flapsClient.Get(ctx, machineID)
+		if err != nil {
+			return fmt.Errorf("could not get machine %s: %w", machineID, err)
+		}
+
+		m, err = machine.AcquireLease(ctx, m)
+		if err != nil {
+			return err
+		}
+		defer flapsClient.ReleaseLease(ctx, m.ID, m.LeaseNonce)
+
+		if err := machine.Restart(ctx, m, input); err != nil {
+			return fmt.Errorf("failed to restart machine %s: %w", m.ID, err)
+		}
+	}
+
+	return nil
 }
